#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "SDL.h"
#include "ui.h"

extern void UpdateGame(unsigned long ulButtons, unsigned long ulJoystick);
extern void RenderScreen(unsigned short *pus);

static unsigned short g_usScreen[320*240];
static SDL_Surface *pScreen = NULL;

static void *InitScreenSDL()
{
	SDL_Init(SDL_INIT_VIDEO);
	pScreen = SDL_SetVideoMode(320, 240, 32, SDL_SWSURFACE);
	memset(g_usScreen, 0, sizeof(g_usScreen));
	return g_usScreen;
}

static void *FlipScreenSDL()
{
	unsigned long ul, r, g, b;
	unsigned short us;
	unsigned long *ptr = (unsigned long *) pScreen->pixels;
	int x, y;

	for(y = 0; y < 240; y++)
	{
		for(x = 0; x < 320; x++)
		{
			us = g_usScreen[y * 320 + x];
			r = (us >> 11) & 0x1F;
			g = (us >> 5) & 0x3F;
			b = us & 0x1F;
			ul = ((r * 8) << 16) + ((g * 4) << 8) + (b * 8);
			ptr[y * (pScreen->pitch / 4) + x] = ul;
		}
	}
	SDL_UpdateRect(pScreen, 0, 0, 320, 240);
	return g_usScreen;
}

static void CleanupScreenSDL()
{
	SDL_Quit();
}

static int g_bUp = 0;
static int g_bDown = 0;
static int g_bLeft = 0;
static int g_bRight = 0;
static int g_bL = 0;
static int g_bR = 0;
static int g_bStart = 0;
static int g_bSelect = 0;
static int g_bX = 0;
static int g_bY = 0;
static int g_bA = 0;
static int g_bB = 0;

unsigned long dzzGetButtons()
{
	unsigned long ul = 0;

	SDL_Event event;

	while(SDL_PollEvent(&event))
	{
		switch(event.type)
		{
			case SDL_KEYDOWN:
				if(event.key.keysym.sym == 'a')
					g_bLeft = 1;
				if(event.key.keysym.sym == 'd')
					g_bRight = 1;
				if(event.key.keysym.sym == 'w')
					g_bUp = 1;
				if(event.key.keysym.sym == 'x')
					g_bDown = 1;
				if(event.key.keysym.sym == 'q')
					g_bL = 1;
				if(event.key.keysym.sym == 'n')
					g_bR = 1;
				if(event.key.keysym.sym == SDLK_ESCAPE)
					g_bX = 1;
				if(event.key.keysym.sym == 'y')
					g_bY = 1;
				if(event.key.keysym.sym == '1')
					g_bA = 1;
				if(event.key.keysym.sym == '2')
					g_bB = 1;
				if(event.key.keysym.sym == ' ')
					g_bStart = 1;
				if(event.key.keysym.sym == 'h')
					g_bSelect = 1;
				break;
			case SDL_KEYUP:
				if(event.key.keysym.sym == 'a')
					g_bLeft = 0;
				if(event.key.keysym.sym == 'd')
					g_bRight = 0;
				if(event.key.keysym.sym == 'w')
					g_bUp = 0;
				if(event.key.keysym.sym == 'x')
					g_bDown = 0;
				if(event.key.keysym.sym == ' ')
					g_bL = 0;
				if(event.key.keysym.sym == 'n')
					g_bR = 0;
				if(event.key.keysym.sym == SDLK_ESCAPE)
					g_bX = 0;
				if(event.key.keysym.sym == 'y')
					g_bY = 0;
				if(event.key.keysym.sym == '1')
					g_bA = 0;
				if(event.key.keysym.sym == '2')
					g_bB = 0;
				if(event.key.keysym.sym == ' ')
					g_bStart = 0;
				if(event.key.keysym.sym == 'h')
					g_bSelect = 0;
				break;
		}
	}
	if(g_bL)
		ul |= GP2X_L;
	if(g_bR)
		ul |= GP2X_R;
	if(g_bStart)
		ul |= GP2X_START;
	if(g_bSelect)
		ul |= GP2X_SELECT;
	if(g_bX)
		ul |= GP2X_X;
	if(g_bY)
		ul |= GP2X_Y;
	if(g_bA)
		ul |= GP2X_A;
	if(g_bB)
		ul |= GP2X_B;
	return ul;
}

unsigned long dzzGetJoystick()
{
	dzzGetButtons();
	if(g_bLeft & g_bUp)
		return JOYDIR_UL;
	if(g_bRight & g_bUp)
		return JOYDIR_UR;
	if(g_bRight & g_bDown)
		return JOYDIR_DR;
	if(g_bLeft & g_bDown)
		return JOYDIR_DL;
	if(g_bLeft)
		return JOYDIR_L;
	if(g_bRight)
		return JOYDIR_R;
	if(g_bUp)
		return JOYDIR_U;
	if(g_bDown)
		return JOYDIR_D;
	return JOYDIR_NONE;
}


static volatile int g_bUIThreadRunning = 0;
static volatile int g_bExitUIThread = 0;

DWORD WINAPI UIThread(LPVOID p)
{
	unsigned long ulButtons;
	unsigned long ulJoystick;

	g_bUIThreadRunning = 1;
	// set up stuff
	InitScreenSDL();
	
	while(!g_bExitUIThread)
	{
		// do any game updating.  About all we'd do here is
		// autoscroll the map
		ulButtons = dzzGetButtons();
		ulJoystick = dzzGetJoystick();
		UpdateGame(ulButtons, ulJoystick);

		// draw the screen buffer
		// using the memscreen and current state, etc
		RenderScreen(g_usScreen);
		// wait for vertical sync
		Sleep(30);
		// flip the screen
		FlipScreenSDL();
	}
	CleanupScreenSDL();
	g_bUIThreadRunning = 0;
	return 0;
}

void StartUIThread()
{
	DWORD dwTID;
	CreateThread(NULL, 0, UIThread, NULL, 0, &dwTID);
}

void ShutdownUIThread()
{
	g_bExitUIThread = 1;
	while(g_bUIThreadRunning)
	{
		Sleep(10);
	}
}

void dzzStartJoystickTimer()
{
}

int dzzGetJoystickTimer()
{
	return 0;
}

void dzzMicroSleep(int nMicros)
{
	Sleep(nMicros/1000);
}
